@@ -1,7 +1,7 @@
 /**
  * Fonts
 
- * Copyright (C) 2007, 2008  Sylvain Beucler
+ * Copyright (C) 2007, 2008, 2009  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
@@ -20,6 +20,10 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__
 #define WIN32_LEAN_AND_MEAN
 #define _WIN32_IE 0x0401
@@ -37,8 +41,10 @@
 #include "io_util.h"
 #include "paths.h"
 #include "gfx_fonts.h"
+#include "gfx_palette.h"
 #include "vgasys_fon.h"
 #include "init.h"
+#include "log.h"
 
 
 /* Default size was 18 in the original game, but it refers to a
@@ -66,8 +72,7 @@ struct font_color
 };
 static struct font_color font_colors[16];
 
-#if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__
-#else
+#ifdef HAVE_FONTCONFIG
 #include <fontconfig/fontconfig.h>
 /* Get filename for canonical font name 'fontname'. Return NULL if the
    font cannot be found (for correctness, no alternate font will be
@@ -81,14 +86,14 @@ char* get_fontconfig_path(char* fontname)
 
   if (!FcInit())
     {
-      fprintf(stderr, "get_fontconfig_path: cannot initialize fontconfig\n");
+      log_error("get_fontconfig_path: cannot initialize fontconfig");
       return NULL;
     }
 
   p = FcNameParse((FcChar8*)fontname);
   if (p == NULL)
     {
-      fprintf(stderr, "get_fontconfig_path: invalid font pattern: %s\n", fontname);
+      log_error("get_fontconfig_path: invalid font pattern: %s", fontname);
       return NULL;
     }
   /* Grab filename attribute */
@@ -97,13 +102,13 @@ char* get_fontconfig_path(char* fontname)
   FcFontSet *fs = FcFontList (0, p, attr);
   if (fs->nfont == 0)
     {
-      fprintf(stderr, "get_fontconfig_path: no matching font\n");
+      log_error("get_fontconfig_path: no matching font for %s", fontname);
       return NULL;
     }
   if (FcPatternGetString(fs->fonts[0], FC_FILE, 0, &strval) == FcResultTypeMismatch
       || strval == NULL)
     {
-      fprintf(stderr, "get_fontconfig_path: cannot find font filename\n");
+      log_error("get_fontconfig_path: cannot find font filename for %s", fontname);
       return NULL;
     }
 
@@ -176,12 +181,12 @@ int gfx_fonts_init()
  */
 void gfx_fonts_quit(void)
 {
-  if (dialog_font)
+  if (dialog_font != NULL)
     {
       TTF_CloseFont(dialog_font);
       dialog_font = NULL;
     }
-  if (system_font)
+  if (system_font != NULL)
     {
       TTF_CloseFont(system_font);
       system_font = NULL;
@@ -203,8 +208,7 @@ static TTF_Font *load_default_font() {
     {
       rwops = find_resource_as_rwops("LiberationSans-Regular.ttf");
     }
-#if defined _WIN32 || defined __WIN32__ || defined __CYGWIN__
-#else
+#ifdef HAVE_FONTCONFIG
   if (rwops == NULL)
     {
       char *path = get_fontconfig_path("Liberation Sans:style=Regular");
@@ -214,11 +218,15 @@ static TTF_Font *load_default_font() {
   if (rwops == NULL)
     {
       init_set_error_msg("Could not open font 'LiberationSans-Regular.ttf'. I tried:\n"
-			 "- loading from executable's resources\n"
 			 "- loading from '%s'\n"
-			 "- loading from '%s'\n",
-			 "- querying fontconfig",
-			 paths_getpkgdatadir(), paths_getdefaultpkgdatadir());
+			 "- loading from '%s'\n"
+			 "- loading from '%s'\n"
+			 "- loading from executable's resources\n"
+#ifdef HAVE_FONTCONFIG
+			 "- querying fontconfig"
+#endif
+			 ,
+			 paths_getpkgdatadir(), paths_getdefaultpkgdatadir(), paths_getexedir());
       return NULL;
     }
 
@@ -253,16 +261,27 @@ int initfont(char* fontname) {
       strcat(path, "\\");
       strcat(path, filename);
 #else
+#  ifdef HAVE_FONTCONFIG
       path = get_fontconfig_path(fontname);
+#  else
+      path = NULL;
+#  endif
 #endif
-      if (path != NULL)
-	new_font = TTF_OpenFont(path, FONT_SIZE);
+      if (path == NULL)
+	{
+	  log_error("initfont: cannot find '%s'", fontname);
+	}
+      else
+	{
+	  new_font = TTF_OpenFont(path, FONT_SIZE);
+	  if (new_font == NULL)
+	    log_error("TTF_OpenFont: %s", TTF_GetError());
+	}
     }
 
-  if (new_font == NULL) {
-    printf("TTF_OpenFont: %s\n", TTF_GetError());
+  if (new_font == NULL)
     return -1;
-  }
+
 
   /* new_font could be loaded - we can free the previous one */
   TTF_CloseFont(dialog_font);
@@ -299,18 +318,18 @@ setup_font(TTF_Font *font)
 {
   char *familyname = TTF_FontFaceFamilyName(font);
   if(familyname)
-    printf("The family name of the face in the font is: %s\n", familyname);
+    log_info("The family name of the face in the font is: %s", familyname);
   char *stylename = TTF_FontFaceStyleName(font);
   if(stylename)
-    printf("The name of the face in the font is: %s\n", stylename);
-  printf("The font max height is: %d\n", TTF_FontHeight(font));
-  printf("The font ascent is: %d\n", TTF_FontAscent(font));
-  printf("The font descent is: %d\n", TTF_FontDescent(font));
-  printf("The font line skip is: %d\n", TTF_FontLineSkip(font));
+    log_info("The name of the face in the font is: %s", stylename);
+  log_info("The font max height is: %d", TTF_FontHeight(font));
+  log_info("The font ascent is: %d", TTF_FontAscent(font));
+  log_info("The font descent is: %d", TTF_FontDescent(font));
+  log_info("The font line skip is: %d", TTF_FontLineSkip(font));
   if(TTF_FontFaceIsFixedWidth(font))
-    printf("The font is fixed width.\n");
+    log_info("The font is fixed width.");
   else
-    printf("The font is not fixed width.\n");
+    log_info("The font is not fixed width.");
 
   TTF_SetFontStyle(font, TTF_STYLE_BOLD);
 }
@@ -358,15 +377,18 @@ print_text (TTF_Font * font, char *str, int x, int y, int w, SDL_Color /*&*/colo
      conversion (commit 2007-11-01) but this was terribly slow (full
      palette conversion done 5 times per text (for the border effect)
      and per frame). */
-  {
-    SDL_PixelFormat fmt;
-    SDL_Palette pal = {256, cur_screen_palette};
-    Uint32 screen_index;
-    fmt.palette = &pal;
-    screen_index = SDL_MapRGB(&fmt, color.r, color.g, color.b);
-    fmt.palette->colors = GFX_real_pal;
-    SDL_GetRGB(screen_index, &fmt, &(color.r), &(color.g), &(color.b));
-  }
+  if (!truecolor)
+    {
+      SDL_PixelFormat fmt;
+      SDL_Palette pal;
+        pal.ncolors = 256;
+        SDL_Color tmppal[256];
+        gfx_palette_get_phys(tmppal);
+        pal.colors = tmppal;
+      fmt.palette = &pal;
+      Uint32 phys_index = SDL_MapRGB(&fmt, color.r, color.g, color.b);
+      SDL_GetRGB(phys_index, GFX_lpDDSBack->format, &(color.r), &(color.g), &(color.b));
+    }
 
   /* Transparent, low quality - closest to the original engine. */
   /* Besides, we do need a monochrome render, since we're doing nasty
@@ -398,7 +420,8 @@ print_text (TTF_Font * font, char *str, int x, int y, int w, SDL_Color /*&*/colo
 
   if (tmp == NULL)
     {
-      printf("Error rendering text: %s; font is %p\n", TTF_GetError(), font);
+      log_error("Error rendering text: %s; font is %p", TTF_GetError(), font);
+      return;
     }
 
   TTF_SizeUTF8 (font, str, &text_w, &text_h);
@@ -540,7 +563,7 @@ print_text_wrap (char *str, rect* box,
     font = system_font;
   else
     {
-      fprintf(stderr, "Error: unknown font type %d\n", font_type);
+      log_error("Error: unknown font type %d", font_type);
       exit(1);
     }
 
