@@ -1,24 +1,23 @@
 /**
  * System initialization, common to FreeDink and FreeDinkEdit
 
- * Copyright (C) 2007, 2008  Sylvain Beucler
+ * Copyright (C) 2007, 2008, 2009  Sylvain Beucler
 
  * This file is part of GNU FreeDink
 
  * GNU FreeDink is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2, or (at
- * your option) any later version.
+ * published by the Free Software Foundation; either version 3 of the
+ * License, or (at your option) any later version.
 
  * GNU FreeDink is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
 
  * You should have received a copy of the GNU General Public License
- * along with program; see the file COPYING. If not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +29,7 @@
 #include <math.h>
 #include <getopt.h>
 #include <unistd.h> /* chdir */
+#include <xalloc.h>
 
 #include <locale.h>
 #include "gettext.h"
@@ -46,8 +46,9 @@
 #include "dinkvar.h"
 #include "gfx.h"
 #include "gfx_fonts.h"
-#include "gfx_tiles.h"
+#include "gfx_palette.h"
 #include "gfx_sprites.h"
+#include "gfx_tiles.h"
 #include "fastfile.h"
 #include "sfx.h"
 #include "bgm.h"
@@ -63,6 +64,13 @@
 #include <windows.h>
 #endif
 
+#ifdef _PSP
+/* The following is necessary if you're running in PRX format, whose
+   default heap size is only 64kB... */
+#include <pspmoduleinfo.h>
+PSP_HEAP_SIZE_MAX();
+#endif
+
 static int g_b_no_write_ini = 0; // -noini passed to command line?
 static char* init_error_msg = NULL;
 
@@ -139,56 +147,47 @@ print_help (int argc, char *argv[])
 */
 void finiObjects()
 {
-	//wDeviceID = mciGetDeviceID("MCI_ALL_DEVICE_ID"); 
+  g_b_kill_app = 1;
+  
+  if (last_saved_game > 0)
+    {
+      log_info("Modifying saved game.");
+      
+      if (!add_time_to_saved_game(last_saved_game))
+	log_error("Error modifying saved game.");
+      last_saved_game = 0;
+    }
 	
-	if (last_saved_game > 0)
-	{
-		Msg("Modifying saved game.");
-		
-		if (!add_time_to_saved_game(last_saved_game))
-			Msg("Error modifying saved game.");
-	}
+  log_path(/*false*/0);
+  
+  if (sound_on)
+    {
+      //lets kill the cdaudio too
+      bgm_quit();
+      QuitSound();
+    }
 	
-	log_path(/*false*/0);
+  kill_all_scripts_for_real();
+  FastFileFini();
 
-	input_quit();
+  dinkc_quit();
+  dinkini_quit();
 
-	gfx_quit();
+  game_quit();
+  
+  input_quit();
+  
+  gfx_quit();
+  
+  //SDL_QuitSubSystem(SDL_INIT_EVENTTHREAD);	
+  SDL_QuitSubSystem(SDL_INIT_TIMER);
+  
+  SDL_Quit();
 
-	if (sound_on)
-	{
-	//lets kill the cdaudio too
-/* 	if (mciSendString("close all", NULL, 0, NULL) != 0) */
-/* 	{ */
-/* 		Msg("Couldn't close all MCI events.."); */
-/* 		//	return(FALSE); */
-/* 	} */
-	  bgm_quit();
-	  QuitSound();
-	}
-	
-	kill_all_scripts_for_real();
-	FastFileFini();
-
-	g_b_kill_app = 1;
-/* 	ShowWindow(hWndMain, SW_HIDE); */
-/* 	SendMessage(hWndMain, WM_IMDONE, 0,0); */
-	//PostQuitMessage(0);
-
-	dinkc_quit();
-	dinkini_quit();
-
-	game_quit();
-
-	SDL_QuitSubSystem(SDL_INIT_EVENTTHREAD);	
-	SDL_QuitSubSystem(SDL_INIT_VIDEO);
-
-	SDL_QuitSubSystem(SDL_INIT_TIMER);
-
-	paths_quit();
-	
-	if (init_error_msg != NULL)
-	  free(init_error_msg);
+  if (init_error_msg != NULL)
+    free(init_error_msg);
+  
+  paths_quit();
 }
 
 /**
@@ -197,8 +196,7 @@ void finiObjects()
 int initFail(char *message)
 {
   msgbox_init_error(message);
-  finiObjects();
-  return 0; /* used when "return initFail(...);" */
+  return -1; /* used when "return initFail(...);" */
 }
 
 
@@ -211,6 +209,7 @@ static int check_arg(int argc, char *argv[])
   int c;
   char *refdir_opt = NULL;
   char *dmoddir_opt = NULL;
+  int debug_p = 0;
 
   /* Options '-debug', '-game', '-noini', '-nojoy', '-nosound' and
      '-window' (with one dash '-' only) are required to maintain
@@ -239,16 +238,13 @@ static int check_arg(int argc, char *argv[])
     {
       switch (c) {
       case 'd':
-	  debug_mode = 1;
-	  /* TODO: use global path */
-	  remove("dink/debug.txt");
-	  break;
+	debug_p = 1;
+	break;
       case 'r':
 	refdir_opt = strdup(optarg);
 	break;
       case 'g':
 	dmoddir_opt = strdup(optarg);
-	printf("Working directory %s requested.", dmoddir_opt);
 	break;
       case 'h':
 	print_help(argc, argv);
@@ -296,16 +292,34 @@ static int check_arg(int argc, char *argv[])
   free(refdir_opt);
   free(dmoddir_opt);
 
-  Msg("Game directory is '%s'.", paths_getdmoddir());
+  if (debug_p == 1)
+    {
+      char* fullpath = paths_dmodfile("DEBUG.TXT");
+      remove(fullpath);
+      free(fullpath);
+
+      log_debug_on();
+    }
+
+  log_info("Game directory is '%s'.", paths_getdmoddir());
   return 1;
 }
 
 
+/**
+ * What to do if running out of memory
+ */
+void xalloc_die (void) {
+  fprintf(stderr, "Memory exhausted!");
+  finiObjects();
+  abort();
+}
+
 
 /* The goal is to replace freedink and freedinkedit's doInit() by a
    common init procedure. This procedure will also initialize each
    subsystem as needed (eg InitSound) */
-int init(int argc, char *argv[])
+int init(int argc, char *argv[], char* splash_path)
 {
   /** i18n **/
   /* Only using LC_MESSAGES because LC_CTYPE (part of LC_ALL) may
@@ -330,8 +344,7 @@ int init(int argc, char *argv[])
 
 
   if (!check_arg(argc, argv))
-    return 0;
-
+    return -1;
 
   /* Same for this D-Mod's .mo (after options are parsed) */
   char* dmod_localedir = paths_dmodfile("l10n");
@@ -340,6 +353,25 @@ int init(int argc, char *argv[])
   free(dmod_localedir);
 
 
+  /* SDL */
+  /* Init timer subsystem */
+  if (SDL_Init(SDL_INIT_TIMER) == -1) {
+    init_set_error_msg("Timer initialization error: %s\n", SDL_GetError());
+    return initFail(init_error_msg);
+  }
+
+  /* Quits in case we couldn't do it properly first (i.e. attempt to
+     avoid stucking the user in 640x480 when crashing) */
+  atexit(finiObjects);
+
+  /* GFX */
+  if (gfx_init(windowed ? GFX_WINDOWED : GFX_FULLSCREEN,
+	       splash_path) < 0)
+    return initFail(init_error_msg);
+
+  /* Joystick */
+  input_init();
+
   /* Engine */
   /* Start with this initialization as it resets structures that are
      filled in other subsystems initialization */
@@ -348,9 +380,10 @@ int init(int argc, char *argv[])
   dinkini_init();
   dinkc_init();
 
+  /* SFX & BGM */
   if (sound_on) 
     {
-      Msg("Initting sound");
+      log_info("Initting sound");
       if (InitSound() < 0)
 	sound_on = 0;
       else
@@ -359,15 +392,8 @@ int init(int argc, char *argv[])
       if (sound_on)
 	bgm_init();
     }
-
-
-  /* SDL */
-  /* Init timer subsystem */
-  if (SDL_Init(SDL_INIT_TIMER) == -1)
-    {
-      Msg("Timer initialization error: %s\n", SDL_GetError());
-      return 0;
-    }
+  if (cd_inserted)
+    PlayCD(7);
 
 
   SDL_initFramerate(&framerate_manager);
@@ -376,19 +402,14 @@ int init(int argc, char *argv[])
   /* SDL_setFramerate(manager, 83); */
   SDL_setFramerate(&framerate_manager, 60);
 
-
-  int gfx_init_success = 0;
-  if (windowed)
-    gfx_init_success = gfx_init(GFX_WINDOWED);
-  else
-    gfx_init_success = gfx_init(GFX_FULLSCREEN);
-  if (gfx_init_success < 0)
-    return initFail(init_error_msg);
-
-
-  /* Joystick */
-  input_init();
-
+  //dinks normal walk
+  log_info("Loading batch...");
+  load_batch();
+  log_info(" done!");
+  
+  log_info("Loading hard...");
+  load_hard();
+  log_info(" done!");
 
   /* We'll handle those events manually */
   SDL_EventState(SDL_ACTIVEEVENT, SDL_IGNORE);
@@ -407,11 +428,7 @@ int init(int argc, char *argv[])
   /* - SDL_KEYDOWN: we want the keydown events for text input
        (show_console and editor input dialog) */
 
-
-  /* Maybe use SDL_QuitSubSystem instead */
-  atexit(SDL_Quit);
-
-  return 1;
+  return 0;
 }
 
 
